/*
  layeredwidget.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "qgmap.h"
#include <stdio.h>
#include "layeredwidget.h"
#include <qwidget.h>
#include <qpixmap.h>

#include "layeredbase.h"

LayeredWidget::LayeredWidget( LayeredBase *parent, const char *name) :
	QObject((QObject *)parent, name)
{
	parent_widget = parent;
	hidden = false;
}

LayeredBase *LayeredWidget::parentWidget()
{
	return parent_widget;
}

void LayeredWidget::setGeometry(const QRect &rect)
{
	widget_position = rect;
}

void LayeredWidget::setGeometry(int x, int y, int w, int h)
{
	widget_position = QRect(x,y,w,h);
}

const QRect & LayeredWidget::geometry()
{
	return widget_position;
}

int LayeredWidget::x()
{
	return widget_position.x();
}

int LayeredWidget::y()
{
	return widget_position.y();
}

int LayeredWidget::width()
{
	return widget_position.width();
}

int LayeredWidget::height()
{
	return widget_position.height();
}

QPoint LayeredWidget::pos()
{
	return widget_position.topLeft();
}

QSize LayeredWidget::size()
{
	return widget_position.size();
}

void LayeredWidget::resize(int w, int h)
{
	widget_position.setWidth(w);
	widget_position.setHeight(h);
}

void LayeredWidget::resize(QSize const & size)
{
	widget_position.setSize(size);
}

bool LayeredWidget::paintEvent(QPaintEvent *e UNUSED, QPixmap &p UNUSED) {return false;}
void LayeredWidget::showEvent(){}
bool LayeredWidget::keyPressEvent(QKeyEvent *e UNUSED) {return false;}
bool LayeredWidget::keyReleaseEvent(QKeyEvent *e UNUSED) {return false;}
bool LayeredWidget::wheelEvent(QWheelEvent *e UNUSED) {return false;}
bool LayeredWidget::mousePressEvent(QMouseEvent *e UNUSED) {return false;}
bool LayeredWidget::mouseReleaseEvent(QMouseEvent *e UNUSED) {return false;}
bool LayeredWidget::mouseClickEvent(QMouseEvent *e UNUSED) {return false;}
bool LayeredWidget::mouseDoubleClickEvent(QMouseEvent *e UNUSED) {return false;}
bool LayeredWidget::mouseMoveEvent(QMouseEvent *e UNUSED) {return false;}

void LayeredWidget::repaint()
{
	parent_widget->repaint();
}

void LayeredWidget::repaint(int x, int y ,int w, int h)
{
	parent_widget->repaint(x, y, w, h);
}

void LayeredWidget::request(QString const &req, QString const &arg)
{
	if(req == "hide"){
		if(arg != name() && arg != "*")
			return;
		hidden = true;
		return;
	}
	if(req == "show"){
		if(arg != name() && arg != "*")
			return;
		hidden = false;
		return;
	}
}
