/*
  mounter.cpp
  Copyright (C) 2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "mounter.h"
#include <stdio.h>
#include <stdlib.h>
#include <qobject.h>
#include <qwidget.h>
#include <qstring.h>
#include <qfileinfo.h>
#include "root.h"
#include "qgmap.h"

#define MNTBASE "/tmp/qgmap/mnt%0"

Mounter::Mounter(QObject *parent, const char *name)
	: QObject(parent, name)
{
	int i;
	QString cmd;
#ifdef QWS
	int ret;
#endif

	Enter();

//	printf("parent=%p, name=%s\n", parent, name);

	for(i = 0; i < MAXMAPPATH + 2; i++){
		QString dev = QString("/dev/loop%0").arg(i);

		QFileInfo info(dev);
		if(!info.exists()){
			cmd = "mknod "+dev+" b 7 %1; "
				"chmod 660 "+dev+"; "
				"chown root.operator "+dev;
			cmd = Root::RootSh(cmd.arg(i));
//			printf("%s\n", cmd.ascii());
#ifdef QWS
			ret = system(cmd.ascii());
#endif
		}

		info.setFile(QString(MNTBASE).arg(i));
		if(!info.isDir()){
			cmd = "mkdir -p " + QString(MNTBASE).arg(i);
//			printf("%s\n", cmd.ascii());
#ifdef QWS
			ret = system(cmd.ascii());
#endif
		}
	}

	index = 0;
	for(i = 0; i < MAXMAPPATH; i++)
		used[i] = 0;

	unmountAll();
}

QString Mounter::mount(QString img_path, int *err)
{
	int i;
	QString cmd;
	QString fstype;
	QString mnt_point;
	QFileInfo info(img_path);

	Enter();

	if(err) *err = 0;

	/* 存在しない場合はエラー */
	if(!info.exists())
		goto err;

	if(img_path.right(4).lower() == ".iso")
		fstype = "iso9660";
	else if(img_path.right(4).lower() == ".rom")
		fstype = "romfs";
	else
		goto err;

	for(i = 0; i < 10; i++){
		if(used[index] == 0)
			break;
		index = (index + 1) %10;
	}

	if(i == 10)
		goto err;

	mnt_point = QString(MNTBASE).arg(index);
	index = (index + 1) %10;

	cmd = "mount -t %0 -o loop,ro %1 %2";
	cmd = Root::RootSh(cmd.arg(fstype).arg(img_path).arg(mnt_point));

//	printf("%s\n", cmd.ascii());
#ifdef QWS
	if(system(cmd.ascii()))
		goto err;
#endif

	// imgs ディレクトリがなければ NG
	info.setFile(mnt_point+"/imgs/");
	if(!info.isDir()){
		unmount(mnt_point);
		goto err;
	}

	mounted.append(mnt_point);
	return mnt_point;

  err:
	if(err)
		*err = 1;
	return "";

}

int Mounter::unmount(QString mnt_point)
{
	Enter();
	QStringList list = mounted.grep(mnt_point);
	int ret = 0;

	if(list.count() == 0)
		return 1;

	mounted.remove(mnt_point);

	QString cmd = "umount %0";
	cmd = Root::RootSh(cmd.arg(mnt_point));
//	printf("%s\n", cmd.ascii());
#ifdef QWS
	ret = system(cmd.ascii());
#endif
	return ret;
}

void Mounter::unmountAll()
{
	Enter();

	while(mounted.count() > 0){
//		printf("umount %s\n", mounted[0].ascii());
		unmount(mounted[0]);
	}
	mounted.clear();

#ifdef QWS
	int i;
	for (i = 0; i < MAXMAPPATH; i++) {
		QString cmd = "umount " + QString(MNTBASE).arg(i) + " 2> /dev/null";
		cmd = Root::RootSh(cmd);
		while (system(cmd.ascii()) == 0)
			;
	}
#endif
}
